var SampleJavascript = {
    version: "0.0.1",
    messageCount: 0,
    promiseCallbacks: [],
    constants: {
            KEYWORD_CALLBACKID: "callbackID",
            KEYWORD_COMMAND: "command",
            KEYWORD_CATEGORY: "category",
            KEYWORD_PAYLOAD: "payload"
        }
};

SampleJavascript.postMessage = function(msg) {
    var callbackID = "cb" + SampleJavascript.messageCount;
    SampleJavascript.messageCount++;

    msg.callbackID = callbackID;

    var promise = new Promise(function(resolve, reject) {
        var callback = {
            callbackID: callbackID,
            resolve: function(value) {
                delete SampleJavascript.promiseCallbacks[callbackID];
                resolve(value);
            },
            reject: function(value) {
                delete SampleJavascript.promiseCallbacks[callbackID];
                reject(value);
            }
        };

        SampleJavascript.promiseCallbacks[callbackID] = callback;

        JSBridge.callNative(JSON.stringify(msg));
    });

    return promise;
}

SampleJavascript.completePromise = function(data) {
    var callbackID = data.callbackID;
    var payload = data.payload;
    var verb = data.verb;

    var cb = SampleJavascript.promiseCallback(callbackID);

    if (typeof cb != 'undefined') {
        if (verb == 'resolve') {
            cb.resolve(payload);
        } else {
            cb.reject(payload);
        }
    }
};

SampleJavascript.promiseCallback = function(callbackID) {
    return SampleJavascript.promiseCallbacks[callbackID];
};
// Function to invoke a Promise based JS function, given the invocation info.
// promiseInvocation is an object with two properties: functionCallStr and promiseCallbackID.
// This function initiates the Promise based call.  Subsequent to that, based on
// whether the result of the Promise is conveyed back to the native side over
// the bridge.  The promiseCallbackID is the unique identifier on the native side to
// correlate a Promise callback coming from the JS side.
SampleJavascript.callPromiseFunction = function(promiseInvocation) {
    var functionCallStr = promiseInvocation.functionCallStr;
    var promiseCallbackID = promiseInvocation.promiseCallbackID;
    var invalidFunctionCallError = {
    category: "promise",
    command: "reject",
    payload: {
    promiseCallbackID: promiseCallbackID,
    reason: "Invalid function call"
    }
    };

    if (typeof functionCallStr !== "string") {
        SampleJavascript.postMessage(invalidFunctionCallError);

        return;
    }

    var promise = null;

    // Protect against any exception that would get thrown as part of evaluating
    // the JS snippet.
    try {
        // The result is expected to be a Promise.
        promise = eval(functionCallStr);
    } catch (e) {
        invalidFunctionCallError.payload.reason = e;
        SampleJavascript.postMessage(invalidFunctionCallError);
        return;
    }

    // Make sure the evaluation returned a Promise.
    if (!(typeof promise === "object" || typeof promise === "function") || !('then' in promise) || typeof(promise.then) !== "function") {
        SampleJavascript.postMessage(invalidFunctionCallError);
        
        return;
    }
    
    var message = {
    category: "promise",
    payload: {
    promiseCallbackID: promiseCallbackID
    }
    };
    
    // Resolve handler for the Promise.  Conveys the resolution to native side over the bridge.
    var onResolve = function (result) {
        message.command = "resolve";
        message.payload.result = result;
        SampleJavascript.postMessage(message);
    };
    
    // Reject handler for the Promise.  Conveys the rejection to native side over the bridge.
    var onReject = function (reason) {
        message.command = "reject";
        message.payload.reason = reason;
        SampleJavascript.postMessage(message);
    }
    
    // Then method invocation for the Promise to execute next steps.  Mainly to message native side.
    promise.then(onResolve, onReject);
};